;;;; -*- Mode: Lisp; Syntax: ANSI-Common-Lisp; Base: 10 -*-
;;;; *************************************************************************
;;;; FILE IDENTIFICATION
;;;;
;;;; Name:          lists.lisp
;;;; Purpose:       Functions for lists for KMRCL package
;;;; Programmer:    Kevin M. Rosenberg
;;;; Date Started:  Apr 2000
;;;;
;;;; $Id: lists.lisp,v 1.1 2003/04/29 00:26:21 kevin Exp $
;;;;
;;;; This file, part of KMRCL, is Copyright (c) 2002 by Kevin M. Rosenberg
;;;;
;;;; KMRCL users are granted the rights to distribute and use this software
;;;; as governed by the terms of the Lisp Lesser GNU Public License
;;;; (http://opensource.franz.com/preamble.html), also known as the LLGPL.
;;;; *************************************************************************

(in-package :kmrcl)


(defun mklist (obj)
  "Make into list if atom"
  (if (listp obj) obj (list obj)))

(defun filter (fn lst)
  "Filter a list by function, eliminate elements where fn returns nil"
  (let ((acc nil))
    (dolist (x lst)
      (let ((val (funcall fn x)))
        (if val (push val acc))))
    (nreverse acc)))

(defun appendnew (l1 l2)
  "Append two lists, filtering out elem from second list that are already in first list"
  (dolist (elem l2)
    (unless (find elem l1)
      (setq l1 (append l1 (list elem)))))
  l1)



(defun remove-tree-if (pred tree)
  "Strip from tree of atoms that satistify predicate"
  (if (atom tree)
      (unless (funcall pred tree)
	tree)
    (let ((car-strip (remove-tree-if pred (car tree)))
	  (cdr-strip (remove-tree-if pred (cdr tree))))
      (cond
       ((and car-strip (atom (cadr tree)) (null cdr-strip))
	(list car-strip))
       ((and car-strip cdr-strip)
	(cons car-strip cdr-strip))
       (car-strip
	car-strip)
       (cdr-strip
	cdr-strip)))))

(defun find-tree (sym tree)
  "Finds an atom as a car in tree and returns cdr tree at that positions"
  (if (or (null tree) (atom tree))
      nil
    (if (eql sym (car tree))
	(cdr tree)
      (aif (find-tree sym (car tree))
	  it
	(aif (find-tree sym (cdr tree))
	    it
	    nil)))))

;;; Keyword functions

(defun remove-keyword (key arglist)
  (loop for sublist = arglist then rest until (null sublist)
	for (elt arg . rest) = sublist
	unless (eq key elt) append (list elt arg)))

(defun remove-keywords (key-names args)
  (loop for ( name val ) on args by #'cddr
	unless (member (symbol-name name) key-names 
		       :key #'symbol-name :test 'equal)
	append (list name val)))


